from plone.uuid.interfaces import IUUID
from zope.publisher.browser import BrowserView


class UUIDView(BrowserView):
    """A simple browser view that renders the UUID of its context"""

    def __call__(self):
        return str(IUUID(self.context, ""))



from plone.uuid.interfaces import ATTRIBUTE_NAME
from plone.uuid.interfaces import IAttributeUUID
from plone.uuid.interfaces import IUUIDGenerator
from zope.component import adapter
from zope.component import queryUtility
from zope.lifecycleevent.interfaces import IObjectCopiedEvent
from zope.lifecycleevent.interfaces import IObjectCreatedEvent


try:
    from Acquisition import aq_base
except ImportError:

    def aq_base(obj):
        # soft-dependency on Zope2, fallback
        return obj


@adapter(IAttributeUUID, IObjectCreatedEvent)
def addAttributeUUID(obj, event):
    if not IObjectCopiedEvent.providedBy(event):
        if getattr(aq_base(obj), ATTRIBUTE_NAME, None):
            return  # defensive: keep existing UUID on non-copy create

    generator = queryUtility(IUUIDGenerator)
    if generator is None:
        return

    uuid = generator()
    if not uuid:
        return

    setattr(obj, ATTRIBUTE_NAME, uuid)



from plone.uuid.interfaces import IUUIDGenerator
from zope.interface import implementer

import uuid


@implementer(IUUIDGenerator)
class UUID4Generator:
    """Default UUID implementation.

    Uses uuid.uuid4()
    """

    def __call__(self):
        return uuid.uuid4().hex



from plone.uuid import interfaces
from zope import component
from zope import interface


@interface.implementer(interfaces.IUUID)
@component.adapter(interfaces.IAttributeUUID)
def attributeUUID(context):
    return getattr(context, interfaces.ATTRIBUTE_NAME, None)


@interface.implementer(interfaces.IMutableUUID)
@component.adapter(interfaces.IAttributeUUID)
class MutableAttributeUUID:
    def __init__(self, context):
        self.context = context

    def get(self):
        return getattr(self.context, interfaces.ATTRIBUTE_NAME, None)

    def set(self, uuid):
        uuid = str(uuid)
        setattr(self.context, interfaces.ATTRIBUTE_NAME, uuid)



from zope.interface import Interface


ATTRIBUTE_NAME = "_plone.uuid"


class IUUIDGenerator(Interface):
    """Utility for generating UUIDs"""

    def __call__():
        """Generate a new UUID."""


class IUUIDAware(Interface):
    """Marker interface for objects that have UUIDs. These should be
    adaptable to IUUID.
    """


class IAttributeUUID(IUUIDAware):
    """Marker interface for objects that have UUIDs stored in a simple
    attribute.

    This interface also confers an event handler that will add UUIDs when
    objects are created (IObjectCreatedEvent).
    """


class IUUID(Interface):
    """Abstract representation of a UUID.

    Adapt an object to this interface to obtain its UUID. Adaptation will
    fail if the object does not have a UUID (yet).
    """


class IMutableUUID(Interface):
    """Adapt an object to this interface to manage the UUID of an object

    Be sure of what you are doing. UUID is supposed to be stable and
    widely used
    """

    def get():
        """Return the UUID of the context"""

    def set(uuid):
        """Set the unique id of the context with the uuid value."""



import unittest


class TestUUID(unittest.TestCase):
    def setUp(self):
        from zope.configuration import xmlconfig

        import plone.uuid
        import zope.component.testing

        zope.component.testing.setUp()
        xmlconfig.file("configure.zcml", plone.uuid)

    def tearDown(self):
        import zope.component.testing

        zope.component.testing.tearDown()

    def test_default_generator(self):
        from plone.uuid.interfaces import IUUIDGenerator
        from zope.component import getUtility

        generator = getUtility(IUUIDGenerator)

        uuid1 = generator()
        uuid2 = generator()

        self.assertNotEqual(uuid1, uuid2)
        self.assertTrue(isinstance(uuid1, str))
        self.assertTrue(isinstance(uuid2, str))

    def test_attribute_uuid_not_set(self):
        from plone.uuid.interfaces import IAttributeUUID
        from plone.uuid.interfaces import IUUID
        from zope.interface import implementer

        @implementer(IAttributeUUID)
        class Context:
            pass

        context = Context()

        uuid = IUUID(context, None)
        self.assertEqual(uuid, None)

    def test_attribute_uuid_create_handler(self):
        from plone.uuid.interfaces import ATTRIBUTE_NAME
        from plone.uuid.interfaces import IAttributeUUID
        from plone.uuid.interfaces import IUUID
        from zope.event import notify
        from zope.interface import implementer
        from zope.lifecycleevent import ObjectCopiedEvent
        from zope.lifecycleevent import ObjectCreatedEvent

        @implementer(IAttributeUUID)
        class Context:
            pass

        context = Context()
        notify(ObjectCreatedEvent(context))

        uuid = IUUID(context, None)
        self.assertNotEqual(uuid, None)
        self.assertTrue(isinstance(uuid, str))

        # calling handler again won't change if UUID already present:
        notify(ObjectCreatedEvent(context))
        self.assertEqual(uuid, IUUID(context, None))

        # ...except when the UUID attribute was the result of a copy
        copied = Context()
        setattr(copied, ATTRIBUTE_NAME, IUUID(context, None))
        self.assertNotEqual(IUUID(copied, None), None)  # mimic copied state
        self.assertEqual(uuid, IUUID(copied, None))  # before handler
        notify(ObjectCopiedEvent(copied, original=context))
        self.assertNotEqual(uuid, None)
        self.assertNotEqual(uuid, IUUID(copied, None))  # copy has new UID

    def test_uuid_view_not_set(self):
        from plone.uuid.interfaces import IAttributeUUID
        from zope.component import getMultiAdapter
        from zope.interface import implementer
        from zope.publisher.browser import TestRequest

        @implementer(IAttributeUUID)
        class Context:
            pass

        context = Context()

        request = TestRequest()
        view = getMultiAdapter((context, request), name="uuid")
        response = view()

        self.assertEqual("", response)
        self.assertTrue(isinstance(response, str))

    def test_uuid_view(self):
        from plone.uuid.interfaces import IAttributeUUID
        from plone.uuid.interfaces import IUUID
        from zope.component import getMultiAdapter
        from zope.event import notify
        from zope.interface import implementer
        from zope.lifecycleevent import ObjectCreatedEvent
        from zope.publisher.browser import TestRequest

        @implementer(IAttributeUUID)
        class Context:
            pass

        context = Context()
        notify(ObjectCreatedEvent(context))

        uuid = IUUID(context, None)

        request = TestRequest()
        view = getMultiAdapter((context, request), name="uuid")
        response = view()

        self.assertEqual(str(uuid), response)
        self.assertTrue(isinstance(response, str))

    def test_uuid_mutable(self):
        from plone.uuid import interfaces
        from zope import event
        from zope import interface
        from zope import lifecycleevent

        @interface.implementer(interfaces.IAttributeUUID)
        class Context:
            pass

        context = Context()
        event.notify(lifecycleevent.ObjectCreatedEvent(context))

        mutable = interfaces.IMutableUUID(context)

        uuid1 = mutable.get()
        mutable.set("a uuid to set")
        uuid2 = mutable.get()
        uuid3 = interfaces.IUUID(context)

        self.assertTrue(uuid1 != uuid2)
        self.assertTrue(uuid2 == uuid3)



from setuptools import find_packages
from setuptools import setup


version = "2.0.0"

long_description = open("README.rst").read() + "\n" + open("CHANGES.rst").read() + "\n"

setup(
    name="plone.uuid",
    version=version,
    description="UUIDs for content items",
    long_description=long_description,
    # Get more strings from
    # https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "Programming Language :: Python",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    keywords="plone uuid",
    author="Martin Aspeli",
    author_email="optilude@gmail.com",
    url="https://github.com/plone/plone.uuid",
    license="BSD",
    packages=find_packages(),
    namespace_packages=["plone"],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        "Acquisition",
        "setuptools",
        "zope.component",
        "zope.browserpage",
        "zope.interface",
        "zope.lifecycleevent",
        "zope.publisher",
    ],
    extras_require={
        "test": [
            "zope.configuration",
            "zope.event",
        ]
    },
    entry_points="""
    """,
)
